@@ -1,14 +1,15 @@
-from scipy.stats import binom, norm, chi2, betabinom  # pylint: disable-msg=E0611
+from scipy.stats import binom, norm, chi2, betabinom, nbinom  # pylint: disable-msg=E0611
 from scipy.special import gammaln, polygamma  # pylint: disable-msg=E0611
-from scipy.optimize import minimize
+from scipy.optimize import minimize, minimize_scalar
 import statsmodels.stats.multitest as multi
 import logging
 import numpy as np
 import pandas as pd
 import itertools
+from scipy.stats import chi2_contingency, fisher_exact
 
 # from .decorators import deprecated, debug, experimental
-from ._utils import _filter_function
+from ._utils import _filter_function, _corrected_log2OR
 
 logger = logging.getLogger('isotools')
 
@@ -131,8 +132,8 @@ TESTS = {'betabinom_lr': betabinom_lr_test,
          'proportions': proportion_test}
 
 
-def _check_groups(transcriptome, groups):
-    assert len(groups) == 2, "length of groups should be 2, but found %i" % len(groups)
+def _check_groups(transcriptome, groups, n_groups=2):
+    assert len(groups) == n_groups, f"length of groups should be {n_groups}, but found {len(groups)}"
     # find groups and sample indices
     if isinstance(groups, dict):
         groupnames = list(groups)
@@ -143,11 +144,11 @@ def _check_groups(transcriptome, groups):
     elif all(isinstance(grp, list) for grp in groups):
         groupnames = ['group{i+1}' for i in range(len(groups))]
     else:
-        raise ValueError('groups not found in dataset')
+        raise ValueError('groups not found in dataset (samples must be a str, list or dict)')
     notfound = [sa for grp in groups for sa in grp if sa not in transcriptome.samples]
     if notfound:
         raise ValueError(f"Cannot find the following samples: {notfound}")
-    assert not (groupnames[0] in groupnames[1] or groupnames[1] in groupnames[0]), 'group names must not be contained in other group names'
+    assert all((gn1 not in gn2 for gn1, gn2 in itertools.permutations(groupnames, 2))), 'group names must not be contained in other group names'
     sa_idx = {sa: idx[0] for sa, idx in transcriptome._get_sample_idx().items()}
     grp_idx = [[sa_idx[sa] for sa in grp] for grp in groups]
     return groupnames, groups, grp_idx
@@ -325,6 +326,25 @@ def alternative_splicing_events(self, min_total=100, min_alt_fraction=.1, sample
 
 # summary tables (can be used as input to plot_bar / plot_dist)
 
+# function to optimize (inverse nbinom cdf)
+
+
+def _tpm_fun(tpm_th, n_reads, cov_th=2, p=.8):
+    return (p-nbinom.cdf(n_reads - cov_th, n=cov_th, p=tpm_th * 1e-6))**2
+
+
+def estimate_tpm_threshold(n_reads, cov_th=2, p=.8):
+    '''Estimate the minimum expression level of obervable transcripts at given coverage.
+
+    The function returns the expression level in transcripts per million (TPM), that can be observed
+    at the given sequencing depth.
+
+    :param n_reads: The sequencing depth (total number of reads) for the sample.
+    :param cov_th: The requested minimum number of reads per transcripts.
+    :param p: The probability of a transcript at threshold expression level to be observed.
+    '''
+    return minimize_scalar(_tpm_fun,  bounds=(.01, 1000), args=(n_reads, cov_th, p))['x']
+
 
 def altsplice_stats(self, groups=None, weight_by_coverage=True, min_coverage=2, tr_filter={}):
     '''Summary statistics for novel alternative splicing.
@@ -716,3 +736,136 @@ def rarefaction(self, groups=None, fractions=20, min_coverage=2, tr_filter={}):
     for sa in cov:
         curves[sa] = [(np.random.binomial(n=cov[sa], p=th) >= min_coverage).sum() for th in fractions]
     return pd.DataFrame(curves, index=fractions), total
+
+
+def pairwise_event_test(e1, e2, coverage, test="chi2"):
+    '''
+    performs an independence test among the joint frequencies of two events
+
+    :param e1: event obtained from .find_splice_bubbles()
+    :param e2: event obtained from .find_splice_bubbles()
+    :param transcriptome: the Transcriptome object on which the events were computed
+    :param coverage: list of counts per transcript
+    :param min_dist: minimum distance (in nucleotides) between the two Alternative Splicing Events for the pair to be tested
+    :type min_dist: int
+    :param test: test to be performed. One of ("chi2", "fisher")
+    :type test: str
+    :param sg: segment graph
+    '''
+
+    con_tab = np.zeros((2, 2), dtype=int)
+    tr_ID_tab = np.zeros((2, 2), dtype=object)
+
+    for m, n in itertools.product(range(2), range(2)):
+        tr_IDs = set(e1[m]) & set(e2[n])
+        cov_tr_IDs = [(coverage[trid], trid) for trid in tr_IDs]  # get coverage and tr_ID
+        cov_tr_IDs.sort(reverse=True)  # sort by coverage
+        tr_ID_tab[n, m] = [trid for _, trid in cov_tr_IDs]
+
+        c_mn = sum([cov for cov, _ in cov_tr_IDs])
+        con_tab[n, m] = c_mn
+
+    if test == 'chi2':
+        test_fun = chi2_contingency
+    elif test == 'fisher':
+        test_fun = fisher_exact
+
+    test_res = test_fun(con_tab+.01)  # add some small value (TODO: is this also for fisher test?)
+
+    priA_priB_trID, altA_altB_trID = tr_ID_tab[0, 0], tr_ID_tab[1, 1]
+    priA_altB_trID, altA_priB_trID = tr_ID_tab[1, 0], tr_ID_tab[0, 1]
+    priA_priB, altA_altB = con_tab[0, 0], con_tab[1, 1]
+    priA_altB, altA_priB = con_tab[1, 0], con_tab[0, 1]
+    p_value = test_res[1]
+    test_stat = test_res[0]
+    log2OR = _corrected_log2OR(con_tab)
+
+    # logOR is a measure of the effect size. coordination between the events is either positive or negative.
+
+    return_tuple = (p_value, test_stat, log2OR, priA_priB, priA_altB, altA_priB, altA_altB,
+                    priA_priB_trID, priA_altB_trID, altA_priB_trID, altA_altB_trID)
+
+    return return_tuple
+
+
+def precompute_events_dict(self, event_type=("ES", "5AS", "3AS", "IR", "ME"), min_cov=100, region=None, progress_bar=True):
+    '''
+    Precomputes the evvents_dict, i.e. a dictionary of splice bubbles. Each key is a gene and each value is the splice bubbles
+    object corresponding to that gene.
+    :param region: The region to be considered. Either a string "chr:start-end", or a tuple (chr,start,end). Start and end is optional.
+
+    '''
+
+    events_dict = {}
+
+    for g in self.iter_genes(region=region, progress_bar=progress_bar):
+        sg = g.segment_graph
+        events = sg.find_splice_bubbles(types=event_type)
+        events_dict[g.id] = [e for e in events if g.coverage.sum(axis=0)[e[0]+e[1]].sum() >= min_cov]
+
+    return events_dict
+
+
+def coordination_test(self, samples=None, test="chi2", min_dist=1, min_total=100, min_alt_fraction=.1, min_cov_pair=100,
+                      events_dict={}, event_type=("ES", "5AS", "3AS", "IR", "ME"), query=None, region=None, method="fdr_bh",
+                      progress_bar=True):
+    '''
+    Performs gene_coordination_test on all genes.
+
+    :param samples: Specify the samples that should be considdered in the test.
+            The samples can be provided either as a single group name, a list of sample names, or a list of sample indices.
+    :param test: Test to be performed. One of ("chi2", "fisher")
+    :type test: str
+    :param min_dist: Minimum distance (in nucleotides) between the two Alternative Splicing Events for the pair to be tested
+    :type min_dist: int
+    :param min_total: The minimum total number of reads for an event to pass the filter
+    :type min_total: int
+    :param min_alt_fraction: The minimum fraction of read supporting the alternative
+    :type min_alt_frction: float
+    :param min_cov_pair: the minimum total number of a pair of the joint occurrence of a pair of event for it to be reported in the result
+    :type min_cov_pair: int
+    :param events_dict: a dictionary of splice bubbles. Each key is a gene and each value is the splice bubbles object corresponding to that
+     gene.
+    :param event_type:  A tuple with event types to test. Valid types are ("ES","3AS", "5AS","IR" or "ME", "TSS", "PAS").
+    Default is ("ES", "5AS", "3AS", "IR", "ME")
+    :param query: If provided, query string is evaluated on all genes for filtering
+    :param region: The region to be considered. Either a string "chr:start-end", or a tuple (chr,start,end). Start and end is optional.
+    :param method: The multiple tet adjustment method. Any value allowed by statsmodels.stats.multitest.multipletests (std: Benjamini-Hochberg)
+
+    :return: a Pandas dataframe, where each column corresponds to the p_values, the statistics
+    (the chi squared statistic if the chi squared test is used and the odds-ratio if the Fisher
+    test is used), the log2 OR, the gene Id, the gene name, the type of the first ASE, the type of the second ASE, the
+    starting coordinate of the first ASE, the ending coordinate of the first ASE, the starting
+    coordinate of the second ASE, the ending coordinate of the second ASE,
+    and the four entries of the contingency table.
+    '''
+
+    test_res = []
+
+    if samples is not None:
+        _, _, groups = _check_groups(self, [samples], 1)
+        samples = groups[0]
+
+    for g in self.iter_genes(region=region, progress_bar=progress_bar, query=query):
+
+        try:
+            next_test_res = g.coordination_test(test=test, samples=samples, min_dist=min_dist, min_total=min_total,
+                                                min_alt_fraction=min_alt_fraction, min_cov_pair=min_cov_pair,
+                                                events=events_dict.get(g.id), event_type=event_type)
+            test_res.extend(next_test_res)
+
+        except Exception:
+            logger.error(f"\nError encounter on {print(g)} {g.id}   :  {g.name}.")
+            raise
+
+    col_names = ("gene_id", "gene_name", "strand", "ase1_type", "ase2_type", "ase1_start", "ase1_end",
+                 "ase2_start", "ase2_end", "p_value", "stat", "log2OR", "priA_priB", "priA_altB", "altA_priB",
+                 "altA_altB", "priA_priB_trID", "priA_altB_trID", "altA_priB_trID", "altA_altB_trID")
+
+    res = pd.DataFrame(test_res, columns=col_names)
+
+    adj_p_value = multi.multipletests(res.p_value, method=method)[1]
+
+    res.insert(10, "adj_p_value", adj_p_value)
+
+    return res
