@@ -4,10 +4,12 @@ from scipy.stats import chi2_contingency
 from Bio.Seq import Seq
 import numpy as np
 import copy
+import itertools
 from .splice_graph import SegmentGraph
 from .short_read import Coverage
 from ._transcriptome_filter import SPLICE_CATEGORY
-from ._utils import pairwise
+from ._utils import pairwise, _filter_event
+from ._transcriptome_stats import pairwise_event_test
 
 import logging
 logger = logging.getLogger('isotools')
@@ -416,6 +418,114 @@ class Gene(Interval):
         'Returns a shallow copy of self.'
         return self.__copy__()
 
+    def _find_splice_sites(exons, transcripts):
+        '''Checks whether the splice sites of a new transcript are present in the set of transcripts.
+        avoids the computation of segment graph, which provides the same functionality.
+
+        :param exons: A list of exon tuples representing the transcript
+        :type exons: list
+        :return: boolean array indicating whether the splice site is contained or not'''
+
+        intron_iter = [pairwise(tr['exons']) for tr in transcripts]
+        current = [next(tr) for tr in intron_iter]
+        contained = np.zeros(len(exons)-1)
+        for j, (e1, e2) in enumerate(pairwise(exons)):
+            for i, tr in enumerate(intron_iter):
+                while current[i][0][1] < e1[1]:
+                    try:
+                        current[i] = next(tr)
+                    except StopIteration:
+                        continue
+                if e1[1] == current[i][0][1] and e2[0] == current[i][1][0]:
+                    contained[j] = True
+        return current
+
+    def coordination_test(self, samples=None, test="chi2", min_dist=1, min_total=100, min_alt_fraction=.1,
+                          min_cov_pair=100, events=None, event_type=("ES", "5AS", "3AS", "IR", "ME")):
+        '''Performs pairwise independence test for all pairs of Alternative Splicing Events (ASEs) in a gene.
+
+        For all pairs of ASEs in a gene creates a contingency table and performs an indeppendence test.
+        All ASEs A have two states, pri_A and alt_A, the primary and the alternative state respectivley.
+        Thus, given two events A and B, we have four possible ways in which these events can occur on
+        a transcript, that is, pri_A and pri_B, pri_A and alt_B, alt_A and pri_B, and alt_A and alt_B.
+        These four values can be put in a contingency table and independence, or coordination,
+        between the two events can be tested.
+
+        :param samples: Specify the samples that should be considdered in the test.
+            The samples can be provided either as a single group name, a list of sample names, or a list of sample indices.
+        :param test: Test to be performed. One of ("chi2", "fisher")
+        :type test: str
+        :param min_dist: Minimum distance (in nucleotides) between the two
+            alternative splicing events for the pair to be tested.
+        :type min_dist: int
+        :param min_total: The minimum total number of reads for an event to pass the filter.
+        :type min_total: int
+        :param min_alt_fraction: The minimum fraction of read supporting the alternative.
+        :type min_alt_fraction: float
+        :param min_cov_pair: the minimum total number of a pair of the joint occurrence of a pair of event for it to be reported in the result
+        :type min_cov_pair: int
+        :param events: a splice_bubble object
+        :param event_type:  A tuple with event types to test. Valid types are
+        ("ES", "3AS", "5AS", "IR", "ME", "TSS", "PAS"). Default is ("ES", "5AS", "3AS", "IR", "ME").
+        Not used if the event parameter is already given.
+        :return: A list of tuples (gene_id, gene_name, strand, ASE1_type, ASE2_type,
+        ASE1_start, ASE1_end, ASE2_start, ASE2_end, priA_priB, priA_altB, altA_priB, altA_altB),
+        where each entrance in the tuple corresponds to the p_value, the statistic, the gene name,
+        the type of the first ASE, the type of the second ASE, the starting coordinate of the first ASE,
+        the ending coordinate of the first ASE, the starting coordinate of the second ASE,
+        the ending coordinate of the second ASE, the four entries of the contingency table for each test performed (a column for each),
+        and the four lists of the transcript IDs sorted by coverage (a column for each list).
+        '''
+
+        if samples is None:
+            cov = self.coverage.sum(axis=0)
+        else:
+            try:
+                # Fast mode when testing several genes
+                cov = self.coverage[samples].sum(0)
+            except IndexError:
+                # Fall back to looking up the sample indices
+                from isotools._transcriptome_stats import _check_groups
+                _, _, groups = _check_groups(self._transcriptome, [samples], 1)
+                cov = self.coverage[groups[0]].sum(0)
+
+        sg = self.segment_graph
+
+        if events is None:
+            events = sg.find_splice_bubbles(types=event_type)
+
+        events = [e for e in events if _filter_event(cov, e, min_total=min_total,
+                                                     min_alt_fraction=min_alt_fraction)]
+
+        test_res = []
+
+        for i, j in itertools.combinations(range(len(events)), 2):
+            if sg.events_dist(events[i], events[j]) < min_dist:
+                continue
+            if (events[i][4], events[j][4]) == ("TSS", "TSS") or (events[i][4], events[j][4]) == ("PAS", "PAS"):
+                continue
+
+            attr = pairwise_event_test(events[i], events[j], cov, test=test)  # append to test result
+
+            if sum(attr[3:7]) < min_cov_pair:  # check that the joint occurrence of the two events passes the threshold
+                continue
+
+            coordinate1 = sg._get_event_coordinate(events[i])
+            coordinate2 = sg._get_event_coordinate(events[j])
+
+            attr = (self.id, self.name, self.strand, events[i][4], events[j][4], coordinate1[0],
+                    coordinate1[1], coordinate2[0], coordinate2[1]) + attr
+
+            # events[i][4] is the events[i] type
+            # coordinate1[0] is the starting coordinate of event 1
+            # coordinate1[0] is the ending coordinate of event 1
+            # coordinate2[0] is the starting coordinate of event 2
+            # coordinate2[1] is the ending coordinate of event 2
+
+            test_res.append(attr)
+
+        return test_res
+
     def die_test(self, groups, min_cov=25, n_isoforms=10):
         ''' Reimplementation of the DIE test, suggested by Joglekar et al in Nat Commun 12, 463 (2021):
         "A spatially resolved brain region- and cell type-specific isoform atlas of the postnatal mouse brain"
