@@ -1,7 +1,7 @@
 import numpy as np
 import logging
 from sortedcontainers import SortedDict  # for SpliceGraph
-from ._utils import pairwise, has_overlap
+from ._utils import pairwise, has_overlap, _interval_dist
 from .decorators import deprecated, experimental
 from typing import Union
 
@@ -192,6 +192,7 @@ class SegmentGraph():
 
         :param exons: A list of exon tuples representing the transcript
         :type exons: list
+        :param alternative: list of splice site indices that match other genes
         :return: pair with the squanti category number and the subcategories as list of novel splicing events
             that produce the provided transcript from the transcripts in splce graph
         :rtype: tuple'''
@@ -284,7 +285,7 @@ class SegmentGraph():
             else:
                 altsplice = {'novel exon': [e]}
             j2 = j1
-        elif (is_first and is_last):  # mono-exon TODO: add info wether there is a ref mono exon?
+        elif (is_first and is_last):  # mono-exon (should not overlap a reference monoexon transcript, this is caught earlier)
             altsplice['mono-exon'] = []
             category = 1
         else:  # check splice sites
@@ -307,7 +308,7 @@ class SegmentGraph():
                     dist = min((self[j][1] - e[1] for j in range(j1, j2 + 1)), key=abs)  # the distance to next junction
                     altsplice.setdefault(f"novel {kind}' splice site", []).append((e[1], dist))
                     category = 3
-                elif self[j2][1] < e[1] and not any(j in self._pas for j in range(j1, j2 + 1)):  # exon end is intronic in ref & not overlapping tss
+                elif self[j2][1] < e[1] and not any(j in self._pas for j in range(j1, j2 + 1)):  # exon end is intronic in ref & not overlapping pas
                     site = 'TSS' if is_reverse else 'PAS'
                     altsplice.setdefault(f'novel exonic {site}', []).append((self[j2][1], e[1]))
                     category = max(1, category)
@@ -428,28 +429,44 @@ class SegmentGraph():
                     fuzzy[i] = sorted(shift, key=abs)[0]  # if there are several possible shifts, provide the smallest
         return fuzzy
 
-    def find_splice_sites(self, exons):
+    def find_splice_sites(self, sj):
         '''Checks whether the splice sites of a new transcript are present in the segment graph.
 
-        :param exons: A list of exon tuples representing the transcript
+        :param sj: A list of 2 tuples with the splice site positions
         :type exons: list
         :return: boolean array indicating whether the splice site is contained or not'''
 
-        j = 0
-        sites = np.zeros((len(exons) - 1) * 2, dtype=bool)
-        for i, (e1, e2) in enumerate(pairwise(exons)):
-            while(self[j].end < e1[1]):
-                j += 1
-                if j == len(self):
-                    return sites
-            if self[j].end == e1[1]:
-                sites[i * 2] = True
-            while(self[j].start < e2[0]):
-                j += 1
-                if j == len(self):
-                    return sites
-            if self[j].start == e2[0]:
-                sites[i * 2 + 1] = True
+        sites = np.zeros(len(sj) * 2, dtype=bool)
+        nodes = iter(self)
+        n = next(nodes)
+        splice_junction_starts = {}
+        splice_junction_ends = {}
+        for i, ss in enumerate(sj):
+            splice_junction_starts.setdefault(ss[0], []).append(i)
+            splice_junction_ends.setdefault(ss[1], []).append(i)
+
+        # check exon ends
+        for sjs, idx in sorted(splice_junction_starts.items()):
+            try:
+                while(sjs > n.end):
+                    n = next(nodes)
+                if n.end == sjs and any(self[s].start > n.end for s in n.suc.values()):
+                    for i in idx:
+                        sites[i * 2] = True
+            except StopIteration:
+                break
+        nodes = iter(self)
+        n = next(nodes)
+        # check exon starts
+        for sje, idx in sorted(splice_junction_ends.items()):
+            try:
+                while(sje > n.start):
+                    n = next(nodes)
+                if n.start == sje and any(self[p].end < n.start for p in n.pre.values()):
+                    for i in idx:
+                        sites[i * 2 + 1] = True
+            except StopIteration:
+                break
         return sites
 
     def get_overlap(self, exons):
@@ -544,6 +561,15 @@ class SegmentGraph():
                     esm[covered, e_nr + 1] = True
         return esm
 
+    def get_exonic_region(self):
+        regs = [[self[0].start, self[0].end]]
+        for n in self[1:]:
+            if n.start == regs[-1][1]:
+                regs[-1][1] = n.end
+            else:
+                regs.append([n.start, n.end])
+        return regs
+
     def get_intersects(self, exons):
         '''Computes the splice junction exonic overlap of a new transcript with the segment graph.
 
@@ -872,6 +898,43 @@ class SegmentGraph():
     def __len__(self):
         return len(self._graph)
 
+    def events_dist(self, e1, e2):
+        '''
+        returns the distance (in nucleotides) between two Alternative Splicing Events.
+
+        :param e1: event obtained from .find_splice_bubbles()
+        :param e2: event obtained from .find_splice_bubbles()
+        :param sg: segment graph
+        '''
+
+        # the event begins at the beginning of the first exon and ends at the end of the last exon
+        e1_coor = [self[e1[2]].start, self[e1[3]].end]  # starting and ending coordinates of event 1
+        e2_coor = [self[e2[2]].start, self[e2[3]].end]  # starting and ending coordinates of event 2
+
+        return _interval_dist(e1_coor, e2_coor)
+
+    def _get_node_starting_at(self, coordinate):
+        '''
+        return the index of the node in the splice graph starting at the given coordinate.
+        '''
+        for i, node in enumerate(self):
+            if node.start == coordinate:
+                return i
+
+    def _get_node_ending_at(self, coordinate):
+        '''
+        return the index of the node in the splice graph ending at the given coordinate.
+        '''
+        for i, node in enumerate(self):
+            if node.end == coordinate:
+                return i
+
+    def _get_event_coordinate(self, event):
+        if event[4] in ("TSS", "PAS"):
+            return (self[event[2]].start, self[event[3]].end)
+        else:
+            return (self[event[2]].end, self[event[3]].start)
+
 
 class SegGraphNode(tuple):
     '''A node in a segment graph represents an exonic segment.'''
