@@ -10,7 +10,6 @@ from ._transcriptome_filter import DEFAULT_GENE_FILTER, DEFAULT_TRANSCRIPT_FILTE
 from . import __version__
 logger = logging.getLogger('isotools')
 
-
 # as this class has diverse functionality, its split among:
 # transcriptome.py (this file- initialization and user level basic functions)
 # _transcriptome_io.py (input/output primary data files/tables)
@@ -18,6 +17,7 @@ logger = logging.getLogger('isotools')
 # _trnascriptome_plots.py (plots)
 # _transcriptome_filter.py (gene/transcript iteration and filtering)
 
+
 class Transcriptome:
     '''Contains sequencing data and annotation for Long Read Transcriptome Sequencing (LRTS) Experiments.
     '''
@@ -74,7 +74,8 @@ class Transcriptome:
                 logger.warning('the pickle file seems to contain sample information... extracting refrence')
                 tr = tr._extract_reference()
         else:
-            logger.error('unknown file format %s', file_format)
+            raise ValueError('invalid file format %s of file %s' % (file_format, reference_file))
+        tr.make_index()
         return tr
 
     def save(self, pickle_file=None):
@@ -97,6 +98,7 @@ class Transcriptome:
         pickled_version = tr.infos.get('isotools_version', '<0.2.6')
         if pickled_version != __version__:
             logger.warning('This is isotools version %s, but data has been pickled with version %s, which may be incompatible', __version__, pickled_version)
+        tr.make_index()
         return tr
 
     def save_reference(self, pickle_file=None):
@@ -120,6 +122,7 @@ class Transcriptome:
         for chrom, tree in self.data.items():
             ref_tr.data[chrom] = IntervalTree(Gene(g.start, g.end, {k: v
                                               for k, v in g.data.items() if k in keep}, ref_tr) for g in tree if g.is_annotated)
+        ref_tr.make_index()
         return ref_tr
 
     def make_index(self):
@@ -216,6 +219,14 @@ class Transcriptome:
         '''The list of chromosome names.'''
         return list(self.data)
 
+    def _add_novel_gene(self, chrom, start, end, strand, info, novel_prefix='PB_novel_'):
+        n_novel = self.novel_genes
+        info.update({'chr': chrom, 'ID': f'{novel_prefix}{n_novel+1:05d}', 'strand': strand})
+        g = Gene(start, end, info, self)
+        self.data.setdefault(chrom, IntervalTree()).add(g)
+        self.infos['novel_counter'] += 1
+        return g
+
     def __str__(self):
         return '{} object with {} genes and {} transcripts'.format(type(self).__name__, self.n_genes, self.n_transcripts)
 
@@ -226,7 +237,8 @@ class Transcriptome:
         return (gene for tree in self.data.values() for gene in tree)
 
     # IO: load new data from primary data files
-    from ._transcriptome_io import add_sample_from_bam, remove_samples, add_short_read_coverage, remove_short_read_coverage, collapse_immune_genes
+    from ._transcriptome_io import add_sample_from_bam, add_sample_from_csv, remove_samples, add_short_read_coverage, \
+        remove_short_read_coverage, collapse_immune_genes
 
     # IO: output data as tables or other human readable format
     from ._transcriptome_io import gene_table, transcript_table, chimeric_table, write_gtf, export_alternative_splicing
@@ -235,8 +247,8 @@ class Transcriptome:
     from ._transcriptome_filter import add_qc_metrics, add_filter, remove_filter, iter_genes, iter_transcripts, iter_ref_transcripts
 
     # statistic: differential splicing, alternative_splicing_events
-    from ._transcriptome_stats import die_test, altsplice_test, alternative_splicing_events, rarefaction
+    from ._transcriptome_stats import die_test, altsplice_test, coordination_test, alternative_splicing_events, rarefaction
 
     # statistic: summary tables (can be used as input to plot_bar / plot_dist)
     from ._transcriptome_stats import altsplice_stats, filter_stats, transcript_length_hist, transcript_coverage_hist,\
-        transcripts_per_gene_hist, exons_per_transcript_hist, downstream_a_hist, direct_repeat_hist
+        transcripts_per_gene_hist, exons_per_transcript_hist, downstream_a_hist, direct_repeat_hist, precompute_events_dict
